/*
 *  file:     m395lib.c
 *  revision: 1.0
 *  date:     22/02/00
 *  author:   JG
 * ----------------------------------------------------------------------
 *
 * M395 12-bit DAC M-module with 16x Voltage Outputs
 * APIS Based ANSI-C Library
 *
 * APIS is AcQuisition Technology's Platform Interface Software.
 * The library software in combination with the demo application
 * (refer to source file: m395demo.c) serve as an example for
 * writing application software for the M395.
 * ----------------------------------------------------------------------
 * Copyright 2000 by AcQuisition Technology bv (c)
 * All Rights Reserved
 * Reproduced Under License
 *
 * This source code is the proprietary confidential property of
 * AcQuisition Technology bv, and is provided to the licensee
 * for documentation and educational purposes only.  Reproduction,
 * publication, or any form of distribution to any party other than
 * the licensee is strictly prohibited.
 * ----------------------------------------------------------------------
 * Edition History
 *
 * #    date        Comments                                        by
 * --   --------    ----------------------------------------------  ---
 * 1.0  22/02/00    First release                                   JG
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "m395defs.h"
#include "apis.h"

/* MACRO BOOT_FROM_RAM
 * If the macro BOOT_FROM_RAM is defined as TRUE,
 * the m395_boot routine determines wether
 * the M395 is booting from RAM or ROM.
 * If the module is booting from RAM a firmware image
 * will be downloaded to the module (ms95firm.c must
 * be linked to the application).
 */
#define BOOT_FROM_RAM   0       /* disable booting from RAM */

/*
 * Forward declarations
 */
/* open path to m395 */
int m395_open(APIS_PATH, APIS_HANDLE *);
/* close path to m395 */
void m395_close(APIS_HANDLE);
/* booting m395 */
int m395_boot (APIS_HANDLE);
/* give command to m395 */
int m395_cmd (APIS_HANDLE, UINT16, UINT16 *, int, int);
/* Configure output range of m395 */
int m395_select_range(APIS_HANDLE, UINT16);
/* Select update mode of m395 */
int m395_select_upd_mode(APIS_HANDLE, UINT16);
/* Set output of one channel */
int m395_setchan(APIS_HANDLE, UINT16, UINT16);
/* Set ouptut of all channels with same value */
int m395_setallchan(APIS_HANDLE, UINT16);
/* Updates output of all channels (only with on-demand-update mode) */
int m395_updchan(APIS_HANDLE);

/*
 * Externals
 */
#if BOOT_FROM_RAM
extern const unsigned char ms95firm[];  /* firmware image */
extern const unsigned ms95firm_sz;      /* firmware size in bytes */
#endif

/* 
 * Function:    m395_open
 *
 * Description: Opens a hardware path to M395
 *              
 * Parameters:  APIS_PATH path_id
 *                  module APIS path id
 *              APIS_HANDLE *handle
 *                  pointer to hardware path handle
 *
 * Returns:     APIS error code
 *
 */
int m395_open(APIS_PATH path_id, APIS_HANDLE *handle)
{
    int result;
    
    /* Open path to M395 for A08D16 access */
    result = apis_open(path_id, handle, 0x100, A08D16);
    
    return (result);
} 

/* 
 * Function:    m395_close
 *
 * Description: Close hardware path to M395
 *              
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *
 * Returns:     Nothing
 *
 */
void m395_close(APIS_HANDLE handle)
{
    apis_close(handle);
}

/*
 * Function:    m395_boot
 *
 * Description: Boot the M395 either from RAM or ROM depending on
 *              which one is enabled.
 *
 *              The macro BOOT_FROM_RAM must be defined as TRUE to boot from
 *              RAM else the M395 will boot from ROM.
 *
 *              RAM enabled:
 *                  - Put M395 in reset.
 *                  - Write boot bit in control register this enables
 *                    RAM boot.
 *                  - Download firmware image
 *                  - Put command page in page register
 *                  - Release reset
 *                  - Wait till ready bit is set this means all channels
 *                    are updated.
 *                  - Check if global status is zero when not zero there is
 *                    a hardware failure.
 *
 *              ROM enabled:
 *                  - Put M395 in reset.
 *                  - Put command page in page register
 *                  - Release reset
 *                  - Wait till ready bit is set
 *                  - Check if global status is zero when not zero there is
 *                    a hardware failure.
 *
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *
 * Returns:     0 or -1 in case of hardware failure
 *
 */
int m395_boot (APIS_HANDLE handle)
{
    UINT16 data;
#if BOOT_FROM_RAM
    UINT32 i, j, size;
    UINT8 *ptr;
#endif

    /* Put module in reset */
    apis_write(handle, sizeof(UINT16), CTRLREG, (UINT16)(RESET));

#if BOOT_FROM_RAM
    /* Set boot bit -> booting from RAM */
    apis_write(handle, sizeof(UINT16), CTRLREG, (UINT16)(BOOT));

    ptr = (UINT8 *)&ms95firm[0];    /* get pointer to boot image */
    size = (ms95firm_sz+128)/128;   /* get size in pages */

    /* Download firmware image to dual ported memory */
    for (i = 0; i < size; i++)
    {
        apis_write(handle, sizeof(UINT16), PAGEREG, (UINT16)(i));
        for (j = 0; j < 64; j++)
        {
            data = (UINT16)(*ptr++ << 8);
            data |= (UINT16)*ptr++;
            apis_write(handle, sizeof(UINT16), (2*j), (UINT16)(data));
        }
    }
#endif

    /* Write 0x100 in page register-> command page in dual ported
       memory */
    apis_write(handle, sizeof(UINT16), PAGEREG, (UINT16)(CMDPAGE));

    /* Release module reset */
    apis_write(handle, sizeof(UINT16), CTRLREG, (UINT16)0);

    /* Wait until ready bit is set */
    do
        apis_read(handle, sizeof(UINT16), CTRLREG, &data);
    while ( !( data & RDY));

    /* Read global status */
    apis_read(handle, sizeof(UINT16), GLOBSTAT, &data);

    if ( data == 0)
        return NOERR;  /* M395 ready for usage */
    else
        return -1; /* Hardware failure */
}

/*
 * Function:    m395_cmd
 *
 * Description: Copy command parameters to the parameter field in
 *              shared ram of the module. Execute command and wait
 *              for the command to be executed.
 *              Copy the result parameters and return with the
 *              result code obtained from the firmware.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UINT16 command
 *                  firmware command
 *              void *pArgList
 *                  pointer to the command parameter list
 *              int ni
 *                  number of input command parameters
 *              int no
 *                  number of output command parameters
 *
 *
 * Returns:     the command result code
 *
 */
int m395_cmd (APIS_HANDLE handle, UINT16 command, UINT16 *pArgList,
                int ni, int no)
{
    UINT16 result;
    UINT16 data;
    int i;

    /* Write 0x100 to page register -> command page in dual ported
       memory */
    apis_write(handle, sizeof(UINT16), PAGEREG, (UINT16)(CMDPAGE));

    /* Write input parameter fields */
    for ( i=0; i < ni; i++)
        apis_write(handle, sizeof(UINT16), (C_PRM0+(2*i)),
                   (UINT16)(pArgList[i]));

    /* Write command to command field */
    apis_write(handle, sizeof(UINT16), C_CMD, (UINT16)(command));

    /* Wait till command is executed */
    do
        apis_read( handle, sizeof(UINT16), C_CMD, &data);
    while (data);

    /* Read result from result field */
    apis_read(handle, sizeof(UINT16), C_RES, &result);

    if ( result == 0)
        /* Read output parameters from parameter fields */
        for ( i=0; i < no; i++)
            apis_read( handle, sizeof(UINT16), (C_PRM0+(2*i)), &pArgList[i]);

    return (int)(result);
}

/*
 * Function:    m395_select_range
 *
 * Description: Puts output range in parameter field 0 and puts RANGE command
 *              in command field. Waits till this command is executed and 
 *              returns the result of it.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UINT16 outp_range
 *                  output range for m395
 *
 * Returns:     NOERR on succes and INV_VALUE when invalid value for output
 *              range
 *
 */
int m395_select_range(APIS_HANDLE handle, UINT16 outp_range)
{
    UINT16 data;
    UINT16 result;
    
    /* Check if output value is legal */
    if ( outp_range > 2) 
        return INV_VALUE;
        
    /* Put output range in parameter field 0 */
    apis_write( handle, sizeof(UINT16), C_PRM0, (UINT16)(outp_range));
    
    /* Execute RANGE command */
    apis_write( handle, sizeof(UINT16), C_CMD, (UINT16)(RANGE));
    
    /* Wait till command is executed */
    do {
        apis_read( handle, sizeof(UINT16), C_CMD, &data);
    } while (data);
    
    /* Read result from result field */
    apis_read(handle, sizeof(UINT16), C_RES, &result);
    return (int)(result);
}
    
/*
 * Function:    m395_select_upd_mode
 *
 * Description: Puts update mode in parameter field 0 and puts UMODE command
 *              in command field. Wait till command is executed and returns
 *              the result of it.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UINT16 upd_mode
 *                  update mode
 *
 * Returns:     NOERR on success and INV_VALUE when invalid value for output
 *              range
 */
int m395_select_upd_mode(APIS_HANDLE handle, UINT16 upd_mode)
{
    UINT16 data;
    UINT16 result;
    
    /* Check if update mode is legal */
    if ( upd_mode > 2)
        return INV_VALUE;
    
    /* Put update mode in parameter field 0 */
    apis_write( handle, sizeof(UINT16), C_PRM0, (UINT16)(upd_mode));
    
    /* Execute UMODE command */
    apis_write( handle, sizeof(UINT16), C_CMD, (UINT16)(UMODE));
    
    /* Wait till command is executed */
    do {
        apis_read( handle, sizeof(UINT16), C_CMD, &data);
    } while (data);
    
    /* Read result from result field */
    apis_read(handle, sizeof(UINT16), C_RES, &result);
    return (int)(result);
}

/*
 * Function:    m395_setchan
 *
 * Description: Sets output of one channel. Fill output data register
 *              of channel with output value. If update mode is continuous
 *              the output is immediately adjusted, when update mode is 
 *              on-demand the m395_updchan() must be called to adjust the
 *              output.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UINT16 channel
 *                  channel number
 *              UINT16 outp_value
 *                  output value
 *
 * Returns:     NOERR on success 
 *
 */
int m395_setchan( APIS_HANDLE handle, UINT16 channel, UINT16 value)
{
    apis_write( handle, sizeof(UINT16), ODR0+(2*channel), (UINT16)(value));
    return NOERR;
}

/*
 * Function:    m395_setallchan
 *
 * Description: Sets output of all channels with same value. Fill output data 
 *              register of channels with output value. If update mode is 
 *              continuous the output is immediately adjusted, when update 
 *              mode is on-demand the m395_updchan() must be called to adjust
 *              the output.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UINT16 channel
 *                  channel number
 *              UINT16 outp_value
 *                  output value
 *
 * Returns:     NOERR on success 
 *
 */
int m395_setallchan( APIS_HANDLE handle, UINT16 value)
{
    int i;
    
    for ( i=0; i<16; i++) {
        apis_write( handle, sizeof(UINT16), ODR0+(2*i), (UINT16)(value));
    }
    return NOERR;
}

/*
 * Function:    m395_updchan
 *
 * Description: Checks ODR registers for changes and updates simultaneous
 *              all outputs. This function is only useful if the m395 is
 *              configured for on-demand-update. Which can be done with
 *              the function m395_select_upd_mode().
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              
 * Returns:     NOERR on succes and ILLREQ when update mode is continuous
 *
 */
int m395_updchan( APIS_HANDLE handle)
{
    UINT16 data;
    UINT16 result;
    
    /* Execute LOAD command */
    apis_write( handle, sizeof(UINT16), C_CMD, (UINT16)(LOAD));
        
    /* Wait till command is executed */
    do {
        apis_read( handle, sizeof(UINT16), C_CMD, &data);
    } while (data);
       
    /* Read result from result field */
    apis_read(handle, sizeof(UINT16), C_RES, &result);
    
    if (result) return (int)(result);
    
    /* Execute UPDATE command */
    apis_write( handle, sizeof(UINT16), C_CMD, (UINT16)(UPDATE));
            
    /* Wait till command is executed */
    do {
        apis_read( handle, sizeof(UINT16), C_CMD, &data);
    } while (data);
           
    /* Read result from result field */
    apis_read(handle, sizeof(UINT16), C_RES, &result);
    
    return (int)(result);
}

