/*
 *  file:     modideep.c
 *  revision: 1.2
 *  date:     03/05/00
 *  author:   JG
 * ----------------------------------------------------------------------
 *
 * Routines to read the M-module ID EEPROM of a M-module placed on
 * an APIS supported M-module carrier board.
 *
 * ----------------------------------------------------------------------
 * Copyright 1999, 2000 by AcQuisition Technology B.V. (c)
 * All Rights Reserved
 * Reproduced Under License
 *
 * This source code is the proprietary confidential property of
 * AcQuisition Technology B.V., and is provided to the licensee
 * for documentation and educational purposes only.  Reproduction,
 * publication, or any form of distribution to any party other than
 * the licensee is strictly prohibited.
 * ----------------------------------------------------------------------
 * Edition History
 *
 * #   date      Comments                                        by
 * --  --------  ----------------------------------------------  ---
 * 1.0 22-04-99  initial version                                 sp
 * 1.1 27-07-99  ported to APIS
 * 1.2 03-05-00  Added id_delay() function to improve execution
 *               speed because apis_delay() takes to much time   jg
 *
 */
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "apis.h"            // Changed path - Bobby Schultz (C & H Technologies)
#include "ideeprom.h"					// Changed order of apis.h and ideeprom.h includes - Bobby Schultz

/* Function prototyping */

int get_modid (APIS_HANDLE, void *);            /* Get EEPROM contents */

/* Function prototyping of local functions */

static int id_cmd(APIS_HANDLE,UINT16);          /* EEPROM command cycle */
static int id_rddata(APIS_HANDLE,UINT16 *);     /* EEPROM input cycle */
static int id_read(APIS_HANDLE,UINT16,UINT16*); /* EEPROM read data word */
static void id_delay(APIS_HANDLE);

/* EEPROM definitions */

#define EEPROM      0x00fe  /* Eeprom register offset for word accesses */
#define CS          0x04    /* Chip Select */
#define SK          0x02    /* Serial data clock */
#define DI          0x01    /* Serial data input */
#define DO          0x01    /* Serial data output */
#define BITMASK     0x0100  /* Bit mask */
#define IDREAD      0x0180  /* Reads data stored in memory */

/*
 * get_modid
 *
 * Descripion:  This function reads the contents of the ID EEPROM on
 *              an M-module.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              void *ideep
 *                  pointer to ID eeprom buffer
 *
 * Returns:     0 if successfull else -1
 */
int get_modid (APIS_HANDLE handle, void *ideep)
{
    UINT16 *dpr, reg, data;

    dpr = (UINT16 *)ideep;

    for (reg = 0; reg < 16; reg++) {
        if (id_read(handle, reg, &data) != 0)
            return -1;
        dpr[reg] = data;
    }
    return 0;
}


/* Local functions */

/*
 * id_cmd
 *
 * Description: Execute an ID EEPROM command cycle.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UINT16 cmds
 *                  command
 *
 * Returns:     0
 */
static int id_cmd(APIS_HANDLE handle, UINT16 cmds)
{
    UINT8 idctrl;
    UINT16 i;

    idctrl = CS;
    apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);
    
    for (i = 0; i < 9; i++)
    {
        if (cmds & BITMASK)
            idctrl |= DI;
        else
            idctrl &= ~DI;
        cmds <<= 1;
        apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);
        id_delay(handle);
        idctrl |= SK;
        apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);
        id_delay(handle);
        idctrl &= ~SK;
        apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);
        id_delay(handle);
    }
    return 0;
}

/*
 * id_rddata
 *
 * descripion:  This function will read a data word from the serial
 *              identicication EEPROM, using bit-toggling.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UINT16 *pData
 *                  pointer to data storage
 *
 * Returns:     0 if successfull else -1
 */
static int id_rddata(APIS_HANDLE handle, UINT16 *pData)
{
    UINT8 idctrl;
    UINT16 data, i, iword;

    apis_read(handle, sizeof (UINT16), EEPROM, (UINT16 *)&iword);
    if (iword & DO)
        return -1;

    data = 0;
    for (i = 0; i < 16; i++)
    {
        idctrl = CS | SK;

        apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);
        id_delay(handle);

        data <<= 1;

        apis_read(handle, sizeof (UINT16), EEPROM, (UINT16 *)&iword);
        if (iword & DO)
            data++;

        idctrl &= ~SK;

        apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);
        id_delay(handle);
    }
    apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)0);
    id_delay(handle);

    *pData = data;

    return 0;
}

/*
 * id_read - read word from M-module ID EEPROM
 *
 * Descripion:  This function will write a read-command to the serial
 *              identicication EEPROM and will read and pass the word.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UINT16 reg
 *                  register to read
 *              UINT16 *pData
 *                  pointer to data storage
 *
 * Returns:     0 if successfull else -1
 */
static int id_read(APIS_HANDLE handle, UINT16 reg, UINT16 *pData)
{
    id_cmd(handle, (UINT16)(IDREAD | reg));
    return id_rddata (handle, pData);
}

/*
 * id_delay
 *
 * description: This function will wait a minimum of 250 nanoseconds.
 *              Using dummy reads.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *
 * Returns:     Nothing
 */
static void id_delay(APIS_HANDLE handle)
{
    int i;
    volatile UINT16 dummy1;
    UINT16 dummy;
    
    for (i=0; i<10; i++) { 
        apis_read(handle, sizeof(UINT16), EEPROM, &dummy);
        dummy1 = dummy;
    }
}
